﻿Imports System.Net
Imports System.Xml.Linq

Namespace CustomLocalization
    ''' <summary>
    ''' Reads and writes to the app.xml configuration file.
    ''' </summary>
    Public NotInheritable Class ConfigurationManager
        ''' <summary>
        ''' Contains the application settings.
        ''' </summary>
        Private Shared ReadOnly AppSettings As IDictionary(Of String, String) = New Dictionary(Of String, String)

        ''' <summary>
        ''' The event that is set when the configuration data is finished loading.
        ''' </summary>
        Public Shared Event Loaded As EventHandler

        ''' <summary>
        ''' The event fired when an error occurs retrieving configuration data.
        ''' </summary>
        Public Shared Event [Error] As EventHandler

        ''' <summary>
        ''' True if the configuration data has been loaded, otherwise false. 
        ''' </summary>
        Public Shared IsLoaded As Boolean

        ''' <summary>
        ''' Loads the configuration settings into memory.
        ''' </summary>
        Private Sub New()
        End Sub
        Public Shared Sub LoadSettings()
            Dim client As New WebClient
            AddHandler client.DownloadStringCompleted, AddressOf ClientDownloadStringCompleted
            client.DownloadStringAsync(New Uri("appSettings.xml", UriKind.Relative))
        End Sub

        ''' <summary>
        ''' Retrieves an application setting.
        ''' </summary>
        ''' <param name="key">The key of the setting to retrieve.</param>
        ''' <returns>The application setting.</returns>
        Public Shared Function GetSetting(ByVal key As String) As String
            If Not AppSettings.ContainsKey(key) Then
                Return String.Empty
            End If

            Dim setting As String = AppSettings(key)

            If String.IsNullOrEmpty(setting) Then
                Return String.Empty
            End If

            If setting.Contains("{") Then
                ' Replace variables here.
            End If

            Return setting
        End Function

        ''' <summary>
        ''' Handles the callback for downloading the app.xml file.
        ''' </summary>
        ''' <param name="sender">The source.</param>
        ''' <param name="e">The arguments.</param>
        Private Shared Sub ClientDownloadStringCompleted(ByVal sender As Object, ByVal e As DownloadStringCompletedEventArgs)
            If e.Error IsNot Nothing Then
                RaiseEvent Error(Nothing, EventArgs.Empty)

                Return
            End If

            ' load the data
            Dim config As XDocument

            Try
                config = XDocument.Parse(e.Result)
            Catch e1 As Exception
                RaiseEvent Error(Nothing, EventArgs.Empty)

                Return
            End Try

            Dim appsettings = From x In config.Descendants("AppSettings").Descendants("Add")
                              Select New With {Key .Key = CStr(x.Attribute("Key")), x.Attribute("Value").Value}

            ' add to collection
            If appsettings.Count() > 0 Then
                For Each y In appsettings
                    ConfigurationManager.AppSettings(y.Key) = y.Value
                Next y
            End If

            IsLoaded = True

            RaiseEvent Loaded(Nothing, EventArgs.Empty)
        End Sub
    End Class
End Namespace